@@ -29,18 +29,10 @@ would be appreciated.
 
 """
 
-from __future__ import (absolute_import, division, print_function)
-
 from random import randint
 import warnings
-import six
-from six.moves import filter
-try:                    # Python 3
-    from urllib.parse import (urlencode, urlparse, urlunparse, parse_qs,
-                              ParseResult)
-except ImportError:      # Python 2
-    from urllib import urlencode
-    from urlparse import urlparse, urlunparse, parse_qs, ParseResult
+from urllib.parse import (urlencode, urlparse, urlunparse, parse_qs,
+                          ParseResult)
 from .etree import etree
 from .util import clean_ows_url, testXMLValue, getXMLInteger, Authentication, openURL
 from .fgdc import Metadata
@@ -135,7 +127,8 @@ class WebMapTileService(object):
             raise KeyError("No content named %s" % name)
 
     def __init__(self, url, version='1.0.0', xml=None, username=None, password=None,
-                 parse_remote_metadata=False, vendor_kwargs=None, auth=None):
+                 parse_remote_metadata=False, vendor_kwargs=None, headers=None, auth=None,
+                 timeout=30):
         """Initialize.
 
         Parameters
@@ -158,6 +151,8 @@ class WebMapTileService(object):
             requests.
         auth : owslib.util.Authentication
             Instance of Authentication class to hold username/password/cert/verify
+        timeout : int
+            number of seconds for GetTile request
 
         """
         self.url = clean_ows_url(url)
@@ -169,11 +164,13 @@ class WebMapTileService(object):
         self.version = version
         self.vendor_kwargs = vendor_kwargs
         self._capabilities = None
+        self.headers = headers
         self.auth = auth or Authentication(username, password)
+        self.timeout = timeout or 30
 
         # Authentication handled by Reader
         reader = WMTSCapabilitiesReader(
-            self.version, url=self.url, auth=self.auth)
+            self.version, url=self.url, headers=self.headers, auth=self.auth)
         if xml:  # read from stored xml
             self._capabilities = reader.readString(xml)
         else:  # read from server
@@ -194,7 +191,7 @@ class WebMapTileService(object):
         # TODO: deprecated function. See ticket #453.
         if not self._capabilities:
             reader = WMTSCapabilitiesReader(
-                self.version, url=self.url, auth=self.auth)
+                self.version, url=self.url, headers=self.headers, auth=self.auth)
             # xml = reader.read(self.url, self.vendor_kwargs)
             # self._capabilities = ServiceMetadata(xml)
             self._capabilities = reader.read(self.url, self.vendor_kwargs)
@@ -207,7 +204,8 @@ class WebMapTileService(object):
 
         # serviceIdentification metadata
         serviceident = self._capabilities.find(_SERVICE_IDENTIFICATION_TAG)
-        self.identification = ServiceIdentification(serviceident)
+        if serviceident is not None:
+            self.identification = ServiceIdentification(serviceident)
 
         # serviceProvider metadata
         serviceprov = self._capabilities.find(_SERVICE_PROVIDER_TAG)
@@ -299,19 +297,6 @@ class WebMapTileService(object):
             Column index of tile to request.
         **kwargs : extra arguments
             anything else e.g. vendor specific parameters
-
-        Example
-        -------
-            >>> url = 'http://map1c.vis.earthdata.nasa.gov/wmts-geo/wmts.cgi'
-            >>> wmts = WebMapTileService(url)
-            >>> wmts.buildTileRequest(layer='VIIRS_CityLights_2012',
-            ...                       tilematrixset='EPSG4326_500m',
-            ...                       tilematrix='6',
-            ...                       row=4, column=4)
-            'SERVICE=WMTS&REQUEST=GetTile&VERSION=1.0.0&\
-LAYER=VIIRS_CityLights_2012&STYLE=default&TILEMATRIXSET=EPSG4326_500m&\
-TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
-
         """
 
         if (layer is None):
@@ -342,7 +327,7 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
         request.append(('TILECOL', str(column)))
         request.append(('FORMAT', format))
 
-        for key, value in six.iteritems(kwargs):
+        for key, value in list(kwargs.items()):
             request.append((key, value))
 
         data = urlencode(request, True)
@@ -352,6 +337,21 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
                           tilematrixset=None, tilematrix=None, row=None,
                           column=None, **kwargs):
 
+        # check the validity of the parameters and set reasonable defaults
+        if layer is None:
+            raise ValueError("layer is mandatory (cannot be None)")
+        if style is None:
+            style = list(self[layer].styles.keys())[0]
+        if tilematrixset is None:
+            tilematrixset = sorted(self[layer].tilematrixsetlinks.keys())[0]
+        if tilematrix is None:
+            msg = 'tilematrix (zoom level) is mandatory (cannot be None)'
+            raise ValueError(msg)
+        if row is None:
+            raise ValueError("row is mandatory (cannot be None)")
+        if column is None:
+            raise ValueError("column is mandatory (cannot be None)")
+
         tileresourceurls = []
         for resourceURL in self[layer].resourceURLs:
             if resourceURL['resourceType'] == 'tile':
@@ -361,13 +361,11 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
             # choose random ResourceURL if more than one available
             resindex = randint(0, numres - 1)
             resurl = tileresourceurls[resindex]['template']
-            if tilematrixset:
-                resurl = resurl.replace('{TileMatrixSet}', tilematrixset)
+            resurl = resurl.replace('{TileMatrixSet}', tilematrixset)
             resurl = resurl.replace('{TileMatrix}', tilematrix)
-            resurl = resurl.replace('{TileRow}', row)
-            resurl = resurl.replace('{TileCol}', column)
-            if style:
-                resurl = resurl.replace('{Style}', style)
+            resurl = resurl.replace('{TileRow}', str(row))
+            resurl = resurl.replace('{TileCol}', str(column))
+            resurl = resurl.replace('{Style}', style)
             return resurl
 
         return None
@@ -434,19 +432,6 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
             Column index of tile to request.
         **kwargs : extra arguments
             anything else e.g. vendor specific parameters
-
-        Example
-        -------
-            >>> url = 'http://map1c.vis.earthdata.nasa.gov/wmts-geo/wmts.cgi'
-            >>> wmts = WebMapTileService(url)
-            >>> img = wmts.gettile(layer='VIIRS_CityLights_2012',\
-                                   tilematrixset='EPSG4326_500m',\
-                                   tilematrix='6',\
-                                   row=4, column=4)
-            >>> out = open('tile.jpg', 'wb')
-            >>> bytes_written = out.write(img.read())
-            >>> out.close()
-
         """
         vendor_kwargs = self.vendor_kwargs or {}
         vendor_kwargs.update(kwargs)
@@ -456,7 +441,7 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
             resurl = self.buildTileResource(
                 layer, style, format, tilematrixset, tilematrix,
                 row, column, **vendor_kwargs)
-            u = openURL(resurl, auth=self.auth)
+            u = openURL(resurl, headers=self.headers, auth=self.auth, timeout=self.timeout)
             return u
 
         # KVP implemetation
@@ -482,13 +467,13 @@ TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'
                     base_url = get_verbs[0].get('url')
             except StopIteration:
                 pass
-        u = openURL(base_url, data, auth=self.auth)
+        u = openURL(base_url, data, headers=self.headers, auth=self.auth, timeout=self.timeout)
 
         # check for service exceptions, and return
         if u.info()['Content-Type'] == 'application/vnd.ogc.se_xml':
             se_xml = u.read()
             se_tree = etree.fromstring(se_xml)
-            err_message = six.text_type(se_tree.find('ServiceException').text)
+            err_message = str(se_tree.find('ServiceException').text)
             raise ServiceException(err_message.strip(), se_xml)
         return u
 
@@ -654,7 +639,7 @@ class TileMatrixSetLink(object):
                         if tml.tilematrix in tilematrixlimits:
                             msg = ('TileMatrixLimits with tileMatrix "%s" '
                                    'already exists' % tml.tilematrix)
-                            raise KeyError(msg)
+                            warnings.warn(msg, RuntimeWarning)
                         tilematrixlimits[tml.tilematrix] = tml
 
                 links.append(TileMatrixSetLink(uri, tilematrixlimits))
@@ -753,11 +738,11 @@ class ContentMetadata:
             legendURL = s.find(_STYLE_LEGEND_URL)
             if legendURL is not None:
                 style['legend'] = legendURL.attrib[_HREF_TAG]
-                if 'width' in legendURL.attrib.keys():
+                if 'width' in list(legendURL.attrib.keys()):
                     style['width'] = legendURL.attrib.get('width')
-                if 'height' in legendURL.attrib.keys():
+                if 'height' in list(legendURL.attrib.keys()):
                     style['height'] = legendURL.attrib.get('height')
-                if 'format' in legendURL.attrib.keys():
+                if 'format' in list(legendURL.attrib.keys()):
                     style['format'] = legendURL.attrib.get('format')
 
             keywords = [f.text for f in s.findall(
@@ -795,7 +780,7 @@ class WMTSCapabilitiesReader:
     """Read and parse capabilities document into a lxml.etree infoset
     """
 
-    def __init__(self, version='1.0.0', url=None, un=None, pw=None, auth=None):
+    def __init__(self, version='1.0.0', url=None, un=None, pw=None, headers=None, auth=None):
         """Initialize"""
         self.version = version
         self._infoset = None
@@ -806,6 +791,7 @@ class WMTSCapabilitiesReader:
             if pw:
                 auth.password = pw
         self.auth = auth or Authentication(un, pw)
+        self.headers = headers
 
     def capabilities_url(self, service_url, vendor_kwargs=None):
         """Return a capabilities url
@@ -840,7 +826,7 @@ class WMTSCapabilitiesReader:
 
         # now split it up again to use the generic openURL function...
         spliturl = getcaprequest.split('?')
-        u = openURL(spliturl[0], spliturl[1], method='Get', auth=self.auth)
+        u = openURL(spliturl[0], spliturl[1], method='Get', headers=self.headers, auth=self.auth)
         return etree.fromstring(u.read())
 
     def readString(self, st):
